pro conserv,momentum=momentum,component=component,dir=dir,$
help=help,all=all,offset=offset,_extra=e
common HD_param, str_par
;+
; NAME:
;       CONSERV
;
; PURPOSE: 
;       Plot the time behaviour of conservative quantities such as
;       mass or components of momentum (linear or angular).
;
; CATEGORY:
;       Input routine
;
; CALLING SEQUENCE:
;       CONSERV [, /MOMENTUM] [, COMPONENT=component] [, DIR=dir]
;       [, /HELP] [, /ALL] [, /OFFSET]
; 
; /MOMENTUM: plots one component of the momentum.
;
; COMPONENT=component: if /MOMENTUM is set, specifies which component
; (between 0 and 2) of the momentum must be displayed. Default to 0.
;
; DIR=dir: looks for the file located at 'dir'. Default is '.', and is
; sticky
;
; /HELP: display this help
;
; /ALL: plot everything (mass + 3 components of the momentum)
;
; /OFFSET: substracts the median value of each field before plotting,
; so as to see the residuals.
;
; Note that extra arguments can be given to that procedure, which are
; transmitted to the PLOT procedure that it uses.
;-
if (keyword_set(help)) then begin
    doc_library, 'conserv'
    return
endif
readstruct, dir=dir
dir=str_par.directory
com = "wc "+str_par.directory+"/conslog.dat"
spawn,com,res
nblines=(long(res))(0)
m = dblarr(6,nblines)
openr,1,filepath('conslog.dat',subdir=dir,root_dir='.')
readf,1,m
close,1
!p.multi=0
!x.range=0
!y.range=0
if keyword_set(offset) then begin
    for j=2,5 do begin
        m(j,*)=m(j,*)-median(m(j,*))
    endfor
endif
idx=[2]
yti=["Mass","Momentum 0", "Momentum 1", "Momentum 2"]
if NOT(keyword_set(component)) then component = 0
if keyword_set(momentum) then idx=[3+component]
if keyword_set(all) then idx=[2,5]
if keyword_set(all) then !p.multi=[0,2,2]
for j=min(idx),max(idx) do begin
    plot,m(1,*),m(j,*),xtitle='!6Time',ytitle=yti[j-2],_extra=e
endfor
return
end
