pro desclist, level=level,nb=nb,list=list,verbose=verbose,dir=dir
common HD_param, str_par
;+
; NAME:
;       DESCLIST
;
; PURPOSE: 
;       Returns the list (as a vector of integers) of cpu-grid
;       numbers that exist for the given level list and output
;       number (this latter needs to be a single number, not a
;       vector). If no level number or level numbers list is
;       supplied, all cpu-grids numbers corresponding to output
;       number #nb (defaulting to 0) is supplied. If no cpu-grid is
;       found the list contains only -1.
;
; CATEGORY:
;       Input routine
;
; CALLING SEQUENCE:
;       DESCLIST [,LEVEL=level] [,NB=nb] [,LIST=list]
;       [,VERBOSE=verbose] [,DIR=dir]
;
; LEVEL=level: level is an integer or vector of
; integers which specifies the list of levels for which one wants
; the cpu-grids list. If this keyword is not supplied all the
; cpu-grids for the corresponding output number are looked up.
;
; NB=nb: Cpu-Grids at output number 'nb' are looked up. Default is
; 0. Useful is the run has been restarted with a different refinement
; hierarchy.
;
; LIST=list: upon return list is a vector of integers which contains
; the cpu-grids numbers found, or -1 if no cpu-grid belonging to
; the output 'nb' and the level list has been found.
;
; /VERBOSE: if this keyword is supplied the cpu-grids list is
; written to the standard output.
;
; DIR=dir: this string specifies the directory 'dir' where to
; look for cpu-grids.
;-
MAXDESC = 4096
if ((n_elements(level) EQ 0) AND $
   (n_elements(nb) EQ 0) AND $
   (NOT Arg_Present(list)) AND $
   (n_elements(verbose) EQ 0)) then begin
    doc_library, 'desclist'
    return
endif
if (NOT(keyword_set(nb))) then nb=0
if (n_elements(level) EQ 0) then level=indgen(100)
scalarlevel=0
if (size(level))(0) EQ 0 then scalarlevel=1
level=[level]
if (NOT(keyword_set(dir))) then dir='.'
if (NOT(keyword_set(verbose))) then verbose=0
exist=lonarr(MAXDESC)
readstruct,nb=nb,dir=dir
lev = str_par.sgrid.level
pgr = str_par.sgrid.parent
cgn = str_par.sgrid.cgridnumber
ncpugrid = str_par.sgrid.nb_cpu_grids
list = [0]
for i=0,ncpugrid-1 do begin
    levcg = lev(pgr(i))
    if (([where(level-levcg EQ 0)])(0) NE -1) then list=[list,cgn(i)]
endfor
if scalarlevel then level=level[0]
if (((size(list))(1)) EQ 1) then begin
    if Verbose then print,'No matching cpu-grid found'
    list = [-1]
    return
endif
list=list(1:*)
if Verbose then print,'Cpu-grids that match output & level: ', list
return
end

