pro drawpolarmesh, mesh, grid=grid, circlesonly=circlesonly, radialonly=radialonly, _extra=e
;+
; NAME:
;       DRAWPOLARMESH
;
; PURPOSE:
;       Display a curvy box that corresponds to a mesh boundary in
;       polar coordinates (work on any device).
;
; CATEGORY:
;       Plotting routine
;
; CALLING SEQUENCE:
;
;       DRAWPOLARMESH, mesh [,/GRID] [,/CIRCLESONLY] [,/RADIALONLY]
;
; MESH: the mesh structure from which the box information is
; extracted. If not present or undefined, print this help. Note that
; MESH must contain the following fields: MESH.FIELD (2d array),
; MESH.XRANGE (a non constant real vector, i.e. with different MIN and
; MAX values), MESH.YRANGE (same thing in Y). The XRANGE field has to
; specify the box width in *Azimuth*, while the YRANGE field has to
; specify the box width in *Radius*.
;
; /GRID: if this keyword is set, display the whole grid structure,
; i.e. all the zones interfaces, not only the grid boundary.
;
; /CIRCLESONLY: if this keyword is set, only the mesh circular
; boundaries or interfaces are shown.
;
; /RADIALONLY: if this keyword is set, only the mesh radial boudaries
; or interfaces are shown.
;
; NOTE:
; ----
; Any keyword of the PLOT routine can be added to DRAWPOLARMESH.
;-
help,mesh,/struct,output=test
if ((total(strpos(test,"UNDEFINED"))+(size(test))(1)) NE 0) then begin
   doc_library, 'drawpolarmesh'
    return
endif
fieldpresent = 0
xrangepresent = 0
yrangepresent = 0
if ((total(strpos(test,"FIELD"))+(size(test))(1)) NE 0) then fieldpresent=1
if ((total(strpos(test,"XRANGE"))+(size(test))(1)) NE 0) then xrangepresent=1
if ((total(strpos(test,"YRANGE"))+(size(test))(1)) NE 0) then yrangepresent=1
if NOT(fieldpresent*xrangepresent*yrangepresent) then begin
    print, "Error: DRAWPOLARMESH needs the mesh argument to be a structure that"
    print, "contains at least the fields 'field' (2D array), 'xrange' (2-vector)"
    print, "and 'yrange' (2-vector)"
    return
endif
nx = (size(mesh.field))[1]
ny = (size(mesh.field))[2]
azimuth=(dindgen(nx)+.5)/nx*(mesh.xrange[1]-mesh.xrange[0])+mesh.xrange[0]
radius=(dindgen(ny)+.5)/ny*(mesh.yrange[1]-mesh.yrange[0])+mesh.yrange[0]
stepr=ny-1
stept=nx-1
if (keyword_set(grid)) then stepr=1
if (keyword_set(grid)) then stept=1
if NOT(keyword_set(RADIALONLY)) then begin
    for i=0,ny-1,stepr do begin
        r=radius[i]
        plots,r*cos(azimuth[0]),r*sin(azimuth[0])
        for j=1,nx-1 do begin
            plots,r*cos(azimuth[j]),r*sin(azimuth[j]),/continue,_extra=e
        endfor
    endfor
endif
if NOT(keyword_set(CIRCLESONLY)) then begin
    for j=0,nx-1,stept do begin
        t=azimuth[j]
        plots,radius[0]*cos(t),radius[0]*sin(t)
        for i=1,ny-1 do begin
            plots,radius[i]*cos(t),radius[i]*sin(t),/continue,_extra=e
        endfor
    endfor
endif
return
end



