pro pasteinto, mesh1, mesh2
;+
; NAME:
;       PASTEINTO
;
; PURPOSE:
;       Paste a 2D array into another, coarser one. Arrays are handled
;       through mesh structures which contain the resolution information.
;
; CATEGORY:
;       Array manipulation routine
;
; CALLING SEQUENCE:
;
;       PASTEINTO, mesh1, mesh2
;
; This routine pastes the content of mesh2 into mesh1. Mesh2 is
; unchanged. Both MESH structures have to contain the following
; entries: 
;
; MESH.FIELD: a real 2D array
; MESH.NCMIN: a long int 2 vector, that contains in integer absolute
; coordinates (see code specifications) the x and y value of the
; bottom left corner of the mesh.
; MESH.NCMAX: a long int 2 vector, that contains in integer absolute
; coordinates (see code specifications) the x and y value of the
; top right corner of the mesh.
;
; Note 1: for this routine to work, Mesh2 has to be fully included
; within mesh1. Mesh2 can have the same resolution, or twice higher,
; than Mesh1 along each axis. In the case the resolution is twice
; higher, an appropriate averaging over mesh2 zones is performed
; before pasting it into mesh1.
;
; Note 2: meaningless results are obtained if some values of mesh2 are
; blanked, since they will be attributed to mesh1 in any case. This
; means that it is not recommended to use the PASTEINTO routine on
; arrays obtained with ASLICE on complex shape meshes.
;
; This routine is used by 'SLICESTACK'
;-
help,mesh1,/struct,output=test1
help,mesh2,/struct,output=test2
if (((total(strpos(test1,"UNDEFINED"))+(size(test1))(1)) NE 0) OR $
    ((total(strpos(test2,"UNDEFINED"))+(size(test2))(1)) NE 0)) then begin
    doc_library, 'pasteinto'
    return
endif
szdest=(size(mesh1.field))(1:2)
szsrc=(size(mesh2.field))(1:2)
rdest =(mesh1.ncmax-mesh1.ncmin)/szdest
rsrc  =(mesh2.ncmax-mesh2.ncmin)/szsrc
pos = (mesh2.ncmin-mesh1.ncmin)/rdest
rsz = szsrc*rsrc/rdest
ratio=rdest/rsrc
idx = indgen(szsrc[0]/ratio[0])*ratio[0]+(ratio[0]-1)*.5
idy = indgen(szsrc[1]/ratio[1])*ratio[1]+(ratio[1]-1)*.5
m = interpolate(mesh2.field,idx,idy,/grid)
mesh1.field(pos[0]:pos[0]+rsz[0]-1,pos[1]:pos[1]+rsz[1]-1) = m
return
end
