function map, x
  return, alog(x)
end

pro polar_slice, n=n, radix=radix, component=component,$
                 genlist=genlist, path=path, field, quiet=quiet,$
                 zslice = zslice, max_value=max_value, min_value=min_value,$
                 substract=substract, show=show,_EXTRA=e_e
   if keyword_set (n) then n=n else n=0
   if keyword_set (radix) then radix=radix else radix='density'
   if keyword_set (component) then component=component else component=0
   if keyword_set (genlist) then descriptornumber=descriptornumber else genlist=[1,1,1,1,1,1,1]
   if keyword_set (path) then path=path else path='../out/test_2d/protoplanet'
   if keyword_set (quiet) then verbose = 0 else verbose = 1
   if keyword_set (max_value) then max_value=max_value; else max_value = 1e30
   if keyword_set (min_value) then min_value=min_value; else min_value = -1e30
   if keyword_set (zslice) then zslice=zslice else zslice = 0.0
   if keyword_set (show) then show = 1 else show = 0
   descfile =strcompress('gas'+radix+string(n)+'_*_*.dat', /remove_all)
   strres = findfile (filepath(descfile, ROOT_dir='.', SUBDIR=path))
   if ((size(strres))(0) EQ 0) then begin
       r = dialog_message(generic_filename+' does not exist in '+path, /error, title='Warning')
       return
   endif
   nb = (size(strres))(1)
   f_set = [0]
   x_set = [0]
   y_set = [0]
   thetamin = 1e20
   thetamax = -1e20
   rmin = 1e20
   rmax = -1e20
   for i=nb-1, 0, -1 do begin       ; We need to read the highest first
       current_name = strres(i)
       c1 = strpos(current_name, '_', /reverse_search)
       c2 = strpos(strmid(current_name, 0, c1-1), '_',/reverse_search)
       descnumber = long(strmid(current_name, c2+1))
       read_field, n=n, radix=radix, component=component,$
         descriptornumber=descnumber, path=path, f, theta, r, z, /quiet, bounds
       if (n_elements(substract) NE 0) then begin
           sb = substract
           read_field, n=sb, radix=radix,$
             component=component, descriptornumber=descnumber, path=path,$
             f0, theta, r, z, /quiet, bounds
           f=f-f0
       endif
       print, bounds
       nt = (size(f))(1)
       nr = (size(f))(2)
       if ((zslice GE bounds(4)) AND (zslice LE bounds(5)) AND (genlist(i) NE 0)) then begin
           tt = replicate (1, nr, 1) ## theta
           rr = r ## replicate (1, nt, 1)
           b = where ((tt LT thetamin) OR (tt GT thetamax) $
                      OR (rr LT rmin) OR (rr GT rmax))
           fr = f(b)
           xr = rr(b)*cos(tt(b))
           yr = rr(b)*sin(tt(b))
           f_set = [f_set, fr]
           x_set = [x_set, xr]
           y_set = [y_set, yr]
           thetamin = min(theta)
           thetamax = max(theta)
           rmin = min(r)
           rmax = max(r)
       endif
   endfor
   f_set = f_set(1:*)
   print, 'field max : ',max(f_set)
   print, 'field min : ',min(f_set)
   if (n_elements (max_value) EQ 0) then max_value = max(f_set)
   if (n_elements (min_value) EQ 0) then min_value = min(f_set)
   x_set = x_set(1:*)
   y_set = y_set(1:*)
   r_set = sqrt(x_set^2+y_set^2)
   rmin = min(r_set)
   f_set(where(r_set LT rmin+1e-6)) = min(f_set)
   bmin = where ((f_set LE min_value), count)
   if count NE 0 then f_set(bmin) = min_value
   bmax = where ((f_set GE max_value), count)
   if count NE 0 then f_set(bmax) = max_value
   contour, map(f_set),x_set,y_set,/irr,$
     max=map(max_value),min=map(min_value),_extra=e_e
   if (show EQ 1) then begin
       for i=nb-1, 0, -1 do begin ; We now display the grid limits
           current_name = strres(i)
           c1 = strpos(current_name, '_', /reverse_search)
           c2 = strpos(strmid(current_name, 0, c1-1), '_',/reverse_search)
           descnumber = long(strmid(current_name, c2+1))
           read_field, n=n, radix=radix, component=component,$
             descriptornumber=descnumber, path=path, f, theta, r, z, /quiet, bounds
           if ((zslice GE bounds(4)) AND (zslice LE bounds(5)) AND (genlist(i) NE 0)) then begin
               tmin = bounds(0)
               tmax = bounds(1)
               rmin = bounds(2)
               rmax = bounds(3)
               x = rmax*cos(tmin)
               y = rmax*sin(tmin)
               plots,x,y
               for j=0,1000 do begin
                   theta=(tmax-tmin)*j/1e3+tmin
                   x=rmin*(cos(theta))
                   y=rmin*(sin(theta))
                   plots,x,y,/continue,noclip=0
               endfor
               for j=0,1000 do begin
                   theta=(tmin-tmax)*j/1e3+tmax
                   x=rmax*(cos(theta))
                   y=rmax*(sin(theta))
                   plots,x,y,/continue,noclip=0
               endfor
           endif
       endfor
   endif
return
end
