pro slicestack,nb=nb,dir=dir,level=level,dim=dim,velocity=velocity,$
coordperp=coordperp,field=field,mesh=mesh,help=help,radix=radix,component=component
common HD_param, str_par
;+
; NAME:
;       ASLICE
;
; PURPOSE:
;
;       Plot a  2D slice of a scalar quantity on a set of
;       cpugrids. This slice includes averaged, embedded upper
;       levels. This routine uses ASLICE and PASTEINTO routines.
;
; CATEGORY:
;       Plotting routine
;
; CALLING SEQUENCE:
;
;       SLICESTACK [,DIR=dir] [,NB=nb] [,LEVEL=level] [,COORDPERP=coordperp]
;       [,DIM=dim] [,/HELP] [,RADIX=radix] [,COMPONENT=component]
;       [,FIELD=field] [,MESH=mesh] [,/VELOCITY]
;
; DIR=dir: the directory where to look for the files. Defaults to '.',
; and is sticky.
;
; NB=nb: the output number of the data that must be used. Defaults to
; 0, and is sticky.
;
; LEVEL=level: The level (integer number) for which one wants to
; extract a slice. Defaults to all 0. It cannot be a vector.
;
; COORDPERP=coordperp: the value of the coordinate perpendicular to
; the cut at which one wants to have a slice. Defauts to 0, i.e. if DIM
; is set to its default (see below) the slice is a (X,Y) slice of the
; system at Z=0.
;
; DIM=dim: 'dim' has to be an integer between 0 and 2, and stands 
; for the dimension perpendicular to the slice. It defaults to 2,
; therefore the slice is in the (X,Y) plane for a cartesian grid
; with no coordinates reordering.
;
; /HELP: prints this help.
;
; RADIX=radix: the radix of the file names to be read. Defaults to
; 'density'.
;
; COMPONENT=component: if the radix is 'velocity', specify the
; component number to be read. Defaults to zero.
;
; FIELD=field: the slice is returned in the 2D array of doubles  FIELD
;
; MESH=mesh: the mesh structure that is returned contains more
; information than the simple 2D array field. It not only contains the
; entry MESH.FIELD, but also MESH.NCMIN and MESH.NCMAX, that give the
; integer min and max corner position on the absolute grid (see code
; specifications), and the entries MESH.XRANGE and MESH.YRANGE that
; give the physical range of the grid corners.
;
; NOTE:
; ----
; This routine works correctly only in the case when the embedded
; levels entirely fill their convex enveloppe. In the cases where some
; upper level zones are blanked, the resulting stacked mesh will be
; erroneous.
;
; /VELOCITY: shortcut for "RADIX='velocity'"
;
;-
if keyword_set(help) OR NOT(Arg_present(field) OR Arg_present(mesh)) then begin
    doc_library, 'slicestack'
    return
endif
if NOT(keyword_set(level)) then level=0
if keyword_set(VELOCITY) then radix="velocity"
readstruct,dir=dir,nb=nb
;we first search the max level
g=str_par.sgrid
lmax=g.maxlevel
for lev=lmax,level,-1 do begin
    aslice,mesh=mnew,lev=lev,dim=dim,$
    coordperp=coordperp,/quiet,radix=radix,component=component
    if (lev NE lmax) then begin
        pasteinto,mnew,mold
    endif
    mold=mnew
endfor
field=mold.field
mesh=mold
return
end
