/* 

$Author: masset $

$Id: ExtPot.c,v 5.17 2007/04/18 16:33:13 masset Exp $

$Log: ExtPot.c,v $
Revision 5.17  2007/04/18 16:33:13  masset
*** empty log message ***

Revision 5.16  2007/02/26 20:22:59  masset
Improved potparser.pl to dump potential law in pot.log

Revision 5.15  2006/08/23 23:07:12  masset
*** empty log message ***

Revision 5.14  2006/08/10 18:23:58  masset
*** empty log message ***

Revision 5.13  2006/08/09 21:15:32  masset
*** empty log message ***


This file corresponds to  the implementation of an external potential,
that is to say an  arbitrary potential with an analytic dependence (it
may  be  time dependent),  not  necessarily  solution  of the  Poisson
equation (the system is  not self-gravitating).  This potential can be
either  specified in  the potential  library,  which is  found in  the
'libpot.txt' file. In this case, the parameter file should include the
parameter 'PotentialCode', set to the string identifying the potential
set up  (e.g. 'kepler').  Note that  you can add any  potential law to
this  library  (it  may  contain   up  to  511  different  laws,  edit
includes/potcodes.h to change this maximum value). The syntax is quite
simple, one should simply be  careful not to forget the semicolumn (;)
at the  end of  each line, except  those containing the  potential law
string identifier,  which end with a  columnb (:). At  build time, the
libpot.txt file  is processed by the perl  script 'potparser.pl' which
creates  automatically 'libpot.cx', 'potcode.c',  and 'includes/pot.h'
(and therefore overrides any  manual change to these files). Basically
this  script  does  the  following  :  (1)  it  converts  the  library
libpot.txt  into a  C source,  in the  file libpot.cx;  note  the 'cx'
suffix  instead of  'c'.   This is  meant  to ensure  that  it is  not
included in the  CVS distribution through the use  of wildcards (*.c),
since  this  file can  be  built  using the  files  given  by the  CVS
distribution.   (2)  It  converts  the  law codes  into  integers  (in
includes/pot.h) and  (3) it associates  the integer to each  string in
potcode.c. Note  that the code will  go through this  procedure if and
only if the  boolean parameter 'ExternalPotential' is set  to YES (see
'actual.c'). It defaults  to NO (see 'var.c'). It  is nevertheless set
automatically to YES if  'PotentialCode' is defined (see switch.c), so
there is  no need to  specify explicitely 'ExternalPotential'  in this
case. Note  that you can also  specify manually the  expression of the
potential  in the present  source file,  between the  comments clearly
stating where  your definition  should be included.  In this  case you
want to remove any potential  code from your parameter file, otherwise
the code will detect that the potential law is multiply defined and it
will issue an error. */

/* Note for writing  a potential law in libpot.txt:  all the variables
x,y,z,radius,azimuth  and colatitude  are defined,  regardless  of the
mesh  geometry.  This  enables  one to  use  e.g. 'x'  as a  temporary
variable say  in cylindrical or spherical geometry,  without having to
declare another variable below */

#include "mhd.h"
#include "pot.h"

static boolean NeverTested = TRUE;

void ComputeExternalPotential (fp, t, phi, flag)
     FluidPatch *fp;
     real t;
     real *phi;
     char flag;
{
  long gncell[3], i, j, k, m, stride[3];
  real xp,yp,zp,work1,work2,work3;
  real x,y,z,radius,azimuth,colatitude; /* have to  be used coherently
					   together   with  the  input
					   file  (e.g. do not  use 'x'
					   in  spherical  coordinates,
					   except   as   a  temporary,
					   unitialized variable */
  real pot;
  getgridsize (fp->desc, gncell, stride);
  for (k = 0; k < gncell[2]; k++) { /* ghosts are included */
    for (j = 0; j < gncell[1]; j++) {
      for (i = 0; i < gncell[0]; i++) {
	m = i*stride[0]+j*stride[1]+k*stride[2];
	if (fp->desc->CommSource[m] & flag) {
	  radius     = x  = fp->desc->Center[_Radial_][m];
	  azimuth    = y  = fp->desc->Center[_Azimuthal_][m];
	  colatitude = z  = fp->desc->Center[_Colatitude_][m];
	  pot = -1e29;

	  /* User needs to edit the following line(s) for his own use */
	  
	  /****************************************************************/
	  /*                                                              */
	  /*       Please Edit BELOW and only BELOW this comment          */
	  /*                                                              */
	  /****************************************************************/
	  /* If you want to paste a patch please do it below this comment */
	  /* and remove the present potential set up                      */
	  /****************************************************************/


	  /****************************************************************/
	  /* Use any of the following: x,y,z,radius,azimuth,colatitude,t  */
	  /****************************************************************/
	  /*                                                              */
	  /*       Please Edit ABOVE and only ABOVE this comment          */
	  /*                                                              */
	  /****************************************************************/
	  
	  /* After this line the source file should NOT be edited */
#include "libpot.cx"
	  phi[m] = pot;
	}
      }
    }
  }
}
