#include "mhd.h"

#define PERCENTMARGIN 20
/* The above means that we damp perturbations up to +/- 20% of the
   mesh radial boundaries */

void ApplyStockholmBoundaryConditions (dt) 
real dt;
{
  FluidWork *fw;
  real *center[3], radius, colatitude;
  real *vel[3], *dens, ramp=0.0, lambda, R_in, R_out, x, y;
  real Tau=0.0, rho0, vrad0, vtheta0, Rmin, Rmax;
  real margin, Colmin, Colmax, damp_zone_min, damp_zone_max;
  long i, j, k, m, gncell[3], stride[3], dim;
  boolean correct;
  margin = 0.01*PERCENTMARGIN;
  fw = CurrentFluidPatch;
  getgridsize (fw->desc, gncell, stride);
  for (dim = 0; dim < 3; dim++) {
    center[dim] = fw->desc->Center[dim];
    vel[dim] = fw->Velocity[dim];
  }
  Rmin = corner_min0[_Radial_];
  Rmax = corner_max0[_Radial_];
  Colmin = corner_min0[_Colatitude_];
  Colmax = corner_max0[_Colatitude_];
  damp_zone_min=margin*fabs(Colmin-M_PI/2.0);
  if (ASPECTRATIO < damp_zone_min) damp_zone_min = ASPECTRATIO;
  damp_zone_max=margin*fabs(Colmax-M_PI/2.0);
  if (ASPECTRATIO < damp_zone_max) damp_zone_max = ASPECTRATIO;
  R_in = Rmin*(1.+margin);
  R_out= Rmax/(1.+margin);
  dens = fw->Density;
  for (k = Nghost[2]; k < gncell[2]-Nghost[2]; k++) {
    for (j = Nghost[1]; j < gncell[1]-Nghost[1]; j++) {
      for (i = Nghost[0]; i < gncell[0]-Nghost[0]; i++) {
	m = i*stride[0]+j*stride[1]+k*stride[2];
	radius = center[_Radial_][m];
	colatitude = center[_Colatitude_][m];               
	correct = NO;
	x = y = 0.0;
	if (radius < R_in) {
	  correct = YES;
	  x = (R_in-radius)/(R_in-Rmin);
	}
	if (radius > R_out) {
	  correct = YES;
	  x = (radius-R_out)/(Rmax-R_out);
	}
	if ((CoordType == SPHERICAL) && (_Colatitude_ < NDIM)) {
	  if (fabs(Colmin - colatitude) < damp_zone_min) {
	    correct = YES;
	    y = 1. - (colatitude - Colmin) / damp_zone_min;
	  }
	  if (fabs(Colmax - colatitude) < damp_zone_max) {
	    correct = YES;
	    y = 1. - (Colmax - colatitude) / damp_zone_max;
	  }
	}
	if (correct == TRUE) {
	  ramp = 1.-(1.-x)*(1.-y);
	  Tau = 2.0*M_PI/(keplerian_init(_vazimuth_,radius,colatitude)+OMEGAFRAME);
	  rho0 = keplerian_init(_density_,radius,colatitude);
	  vrad0 = keplerian_init(_vrad_,radius,colatitude);
	  vtheta0 = keplerian_init(_vazimuth_,radius,colatitude); 
	  	  
	  lambda = ramp*ramp/Tau*dt;

	  dens[m] = (dens[m]+lambda*rho0)/(1.+lambda);
	  vel[_Radial_][m] = (vel[_Radial_][m]+lambda*vrad0)/(1.+lambda);
	  vel[_Azimuthal_][m] = (vel[_Azimuthal_][m]+lambda*vtheta0)/(1.+lambda);
	  if (_Colatitude_ < NDIM)
	    vel[_Colatitude_][m] = vel[_Colatitude_][m]/(1.+lambda);
	}
      }
    }
  }
}
