#include "mhd.h"

inline boolean boundary (rho,e,u,v,w,rhog,eg,ug,vg,wg,x,xg,yg,zg,dlt,dltg,condition,line,predictive)
     real rho,e,u,v,w,x,xg,yg,zg,dlt;
     boolean predictive;	/* Predictive is true whenever
     'boundary' is called from within a Riemann problem predictor. It
     is false otherwise (ie when filling the mesh boundaries) */
     real *rhog,*eg,*ug,*vg,*wg,*dltg;
				/* x and xg are the zone coordinate
                                   along dimension perpendicular to
                                   the boundary, respectively for
                                   source and dest (ie ghost) zones */
     long condition,line;		/* u is the perpendicular velocity */
				/* v and w are the transverse velocities */
				/* dlt & dltg are the value for the 'delta' */
				/* solution of the LeVeque problem. Many */
				/* application don't use LeVeque's zone*/
				/* splitting, but it is a good idea to include */
				/* the 'delta' boundary mapping for any case. */
				/* The line below defines a default value for dltg */
{
  boolean Sym = TRUE;
  *dltg = 0.0;
  switch (condition) {
  /* usual conditions */
  case 1:	       /* REFLECTING */
    *rhog = rho;
    *eg   = e;
    *ug   = -u;
    *vg   = v;
    *wg   = w;
    *dltg = -dlt;
    break;
  case 2:	       /* OUTFLOW */
    *rhog = rho;
    *eg   = e;
    *ug   = u;
    *vg   = v;
    *wg   = w;
    *dltg = dlt;
    break;
    /* problem-specific conditions */
  case 10 : // simple keplerian with dim azim < dim z or dim colat
    *rhog = rho;
    *eg = e;
    *ug = u;
    *vg = (v-OMEGAFRAME)*pow(x/xg,1.5)+OMEGAFRAME;
    *wg = w;
    *dltg = dlt;
    break;
  case 11 : // keplerian 2-3D : radius (COORDPERMUT = 123 or 213 or 231)
    if (predictive) break;
    *rhog = keplerian_boundary(11,_density_,rho,x,xg);
    *eg = keplerian_boundary(11,_energy_,e,x,xg);
    *ug = keplerian_boundary(11,_vrad_,u,x,xg);
    *vg = keplerian_boundary(11,_vazimuth_,v,x,xg);
    *dltg = dlt;
    *wg = 0;
    break;


  case 14 : // keplerian 3D : radius (COORDPERMUT = 123 or 213 or 231) Radial with LeVeque split
    Sym = FALSE;
    if (predictive) break;
    *rhog = keplerian_init (_density_,xg,zg)/keplerian_init (_density_,x,zg)*rho;
    *eg   = keplerian_init (_energy_,xg,zg);
    *ug   = keplerian_init (_vrad_,xg,zg)/keplerian_init (_vrad_,x,zg)*u;
    *vg   = keplerian_init (_vazimuth_,xg,zg);
    *wg = w;
    *dltg = dlt*x/xg;
    break;

  case 30 :            /* 30 & 31 implemented for local wedge sim's */
    if (predictive) break;
    if (yg < 1.0) {
      *rhog=SIGMA0;
      *eg=ASPECTRATIO*ASPECTRATIO;
      *vg=0.0;
      *wg=0.0;
      *ug=pow(yg,-1.5)-1.0;
    } else {
      *rhog=rho;
      *eg=e;
      *ug=u;
      *vg=v;
      *wg=w;
    }
    break;
  case 31 :
    if (predictive) break;
    if (yg > 1.0) {
      *rhog=SIGMA0;
      *eg=ASPECTRATIO*ASPECTRATIO;
      *vg=0.0;
      *wg=0.0;
      *ug=pow(yg,-1.5)-1.0;
    } else {
      *rhog=rho;
      *eg=e;
      *ug=u;
      *vg=v;
      *wg=w;
    }
    break; 

    
  case 40 :            /* 40 & 41 2D test with radial gradient of pressure  */
    if (predictive) break;
    if (yg < 1.0+(SIGMASLOPE-1.0)*ASPECTRATIO*ASPECTRATIO/3.0) {
      *rhog=keplerian_init(_density_,yg,M_PI/2.0);
      *eg=keplerian_init(_energy_,yg,M_PI/2.0);
      *vg=0.0;
      *wg=0.0;
      *ug=keplerian_init(_vazimuth_,yg,M_PI/2.0);
    } else {
      *rhog=rho;
      *eg=e;
      *ug=u;
      *vg=v;
      *wg=w;
    }
    break;
  case 41 :
    if (predictive) break;
    if (yg > 1.0+(SIGMASLOPE-1.0)*ASPECTRATIO*ASPECTRATIO/3.0) {
      *rhog=keplerian_init(_density_,yg,M_PI/2.0);
      *eg=keplerian_init(_energy_,yg,M_PI/2.0);
      *vg=0.0;
      *wg=0.0;
      *ug=keplerian_init(_vazimuth_,yg,M_PI/2.0);
    } else {
      *rhog=rho;
      *eg=e;
      *ug=u;
      *vg=v;
      *wg=w;
    }
    break; 
    
  case 12 : // keplerian 3D : colatitude (COORDPERMUT = 123)
    if (predictive) break;
    *rhog = keplerian_boundary(12,_density_,rho,x,xg);
    *eg = keplerian_boundary(12,_energy_,e,x,xg);
    *ug = -u;
    *wg = w;
    *vg = keplerian_boundary(12,_vazimuth_,v,x,xg);
    *dltg = dlt;
    break;
  case 15 : // keplerian 3D : radius
    if (predictive) break;
    *rhog = keplerian_boundary(12,_density_,rho,x,xg);
    *eg = keplerian_boundary(12,_energy_,e,x,xg);
    *ug = -u;
    *vg = keplerian_boundary(11,_vazimuth_,w,x,xg);
    *wg = 0.0;
    *dltg = dlt;
    break;
  case 20 : // Specific condition for isothermal 1d atmosphere (isoatm)
            // with no LeVeque's zone splitting
    if (predictive) {
      *rhog = rho;
      *eg = e;
      *ug = -u;
      *vg = v;
      *wg = w;
    } else {
      *rhog = rho*exp(-(xg-x)/e);
      *eg = e;
      *ug = -u;
      *vg = v;
      *wg = w;
    }
    break;
  case 32 :		     /* 32 & 33 implemented for local wedge sim's */
			     /* Correspond to novertgrad initial conditions */
    if (predictive) break;
    if (yg < 1.0) {
      *rhog=1.0/yg/sqrt(yg);
      *eg=ASPECTRATIO*ASPECTRATIO*yg*sqrt(yg);
      *vg=0.0;
      *wg=0.0;
      *ug=pow(yg,-1.5)-1.0;
    } else {
      *rhog=rho;
      *eg=e;
      *ug=u;
      *vg=v;
      *wg=w;
    }
    break;
  case 33 :
    if (predictive) break;
    if (yg > 1.0) {
      *rhog=1.0/yg/sqrt(yg);
      *eg=ASPECTRATIO*ASPECTRATIO*yg*sqrt(yg);
      *vg=0.0;
      *wg=0.0;
      *ug=pow(yg,-1.5)-1.0;
    } else {
      *rhog=rho;
      *eg=e;
      *ug=u;
      *vg=v;
      *wg=w;
    }
    break;

    /* Volatile ultra specific boundary conditions below */

  case 332 :		     /* 32 & 33 implemented for local wedge sim's */
    if (predictive) break;
    if (yg < .999166666667) {
      *rhog=1.0;
      *eg=ASPECTRATIO*ASPECTRATIO/yg;
      *vg=0.0;
      *wg=0.0;
      *ug=pow(yg,-1.5)*sqrt(1.0-ASPECTRATIO*ASPECTRATIO)-OMEGAFRAME;
    } else {
      *rhog=rho;
      *eg=e;
      *ug=u;
      *vg=v;
      *wg=w;
    }
    break;
  case 333 :
    if (predictive) break;
    if (yg >= .999166666667) {
      *rhog=1.0;
      *eg=ASPECTRATIO*ASPECTRATIO/yg;
      *vg=0.0;
      *wg=0.0;
      *ug=pow(yg,-1.5)*sqrt(1.0-ASPECTRATIO*ASPECTRATIO)-OMEGAFRAME;
    } else {
      *rhog=rho;
      *eg=e;
      *ug=u;
      *vg=v;
      *wg=w;
    }
    break;


    /* unknown conditions */
  default:
    prs_error ("Unknow boundary condition code at line %ld in %s",\
	       line, GRIDFILE);
  }
  return Sym;
}

void TrueBC_fp (fluid)
     FluidPatch *fluid;
{
  long gncell[3], stride[3], dim, side, dim1, dim2;
  long i_in, i_gh, j, k, m_in, m_gh, bc, offside;
  real *dens, *vel[3], *energy;
  real foo1=0.0, foo2=0.0, foo3=0.0;
  real vp=0.0, *vpg, vt1=0.0, *vtg1, vt2=0.0, *vtg2;
  /* The above initializations are necessary, otherwise when
     'boundary' is first called below to test the symmetry properties,
     a floating point error can be generated (especially on HP/SC) */
  real dlt,*dltg;
  real x, xg;			/* See use of x & xg in the boundary
  function above */
  real yg, zg;
  tGrid_CPU *cpug;
  boolean Sym;
  cpug = fluid->desc;
  dens = fluid->Density->Field;
  energy = fluid->Energy->Field;
  vtg1 = &foo1;
  vtg2 = &foo2;
  for (j = 0; j < 3; j++)
    vel[j] = fluid->Velocity->Field[j];
  getgridsize (cpug, gncell, stride);
  for (dim = 0; dim < NDIM; dim++) {
    for (side = INF; side <= SUP; side++) {
      if ((bc = cpug->iface[dim][side]) > 0) {	/* True boundary conditions */
	Sym = boundary (vt1,vt2,vp,vt1,vt2,&foo1,&foo2,&foo1,&foo2, \
			&foo3,1.0,1.0,1.0,1.0,0.0,&foo3,bc,cpug->Parent->linenumber, FALSE);
	/* The above line is just meant to know the symmetry
	   properties of the boundary condition */
	dim1 = (dim == 0);
	dim2 = 2-(dim == 2);
	offside = (side == INF ? 0 : gncell[dim]-Nghost[dim]);
	for (i_gh = offside; i_gh < offside+Nghost[dim]; i_gh++) {
	  i_in = (2*Nghost[dim]-1)-i_gh;
	  if (side == SUP) i_in += 2*(gncell[dim]-2*Nghost[dim]);
	  if (!Sym)
	    i_in = (side == INF ? Nghost[dim] : gncell[dim]-Nghost[dim]-1);
	  for (j = 0; j < gncell[dim1]; j++) {
	    for (k = 0; k < gncell[dim2]; k++) {
	      m_in = i_in*stride[dim]+j*stride[dim1]+k*stride[dim2];
	      m_gh = i_gh*stride[dim]+j*stride[dim1]+k*stride[dim2];
	      vp = vel[dim][m_in];
	      vpg=&vel[dim][m_gh];
	      x = fluid->desc->Center[dim][m_in];
	      xg= fluid->desc->Center[dim][m_gh];
	      yg= fluid->desc->Center[dim1][m_gh];
	      zg= fluid->desc->Center[dim2][m_gh];
	      if (NDIM > 1) {
		vt1 = vel[dim1][m_in];
		vtg1=&vel[dim1][m_gh];
	      }
	      if (NDIM > 2) {
		vt2 = vel[dim2][m_in];
		vtg2=&vel[dim2][m_gh];
	      }
	      if (splitdimnb[dim] != -1) {
		dlt = fluid->Delta->Field[splitdimnb[dim]][m_in];
		dltg=&(fluid->Delta->Field[splitdimnb[dim]][m_gh]);
	      } else {
		dlt = 0.0;
		dltg= &foo3;
	      }
	      boundary (dens[m_in],energy[m_in],vp,vt1,vt2,&dens[m_gh],\
			&energy[m_gh],vpg,vtg1,vtg2,x,xg,yg,zg,dlt,dltg, \
			bc,cpug->Parent->linenumber, FALSE);
	    }
	  }
	}
      }
    }
  }
}

void TrueBC (level)
     long level;
{
  tGrid_CPU *item;
  item = Grid_CPU_list;
  while (item != NULL) {
    if ((level == item->level) && (item->cpu == CPU_Rank))
      TrueBC_fp (item->Fluid);
    item = item->next;
  }
}

