#include "mhd.h"

ScalarField *CreateScalarField (desc, name, ptr)
     tGrid_CPU *desc;
     char *name;
     real *ptr;
{
  char *Name;
  char string[MAXLINELENGTH];
  ScalarField *sf;
  long size[3], i;
  real *Field;
  sf = prs_malloc (sizeof(ScalarField));
  if (strlen(name) > MAXNAMELENGTH) {
    sprintf (string, "%s is longer than the %ld characters limit.", name, MAXNAMELENGTH);
    prs_error (string);
  }
  Name = prs_malloc(sizeof(char)*(MAXNAMELENGTH+1));
  strcpy (Name, name);
  sf->Name = Name;
  sf->desc = desc;
  for (i = 0; i < 3; i++)
    size[i] = desc->gncell[i];
  Field = ptr;
  sf->Field = Field;
  return sf;
}

void FreeScalarField (sf)
     ScalarField *sf;
{
  free (sf->Name);
  free (sf);
}

MultiScalarField *CreateMultiScalarField (desc, name, n, ptr)
     tGrid_CPU *desc;
     char *name;
     long n;
     real *ptr;
{
  char *Name;
  char string[MAXLINELENGTH];
  MultiScalarField *sf;
  long size[3], i;
  real *Field;
  sf = prs_malloc (sizeof(MultiScalarField));
  if (strlen(name) > MAXNAMELENGTH) {
    sprintf (string, "%s is longer than the %ld characters limit.", name, MAXNAMELENGTH);
    prs_error (string);
  }
  Name = prs_malloc(sizeof(char)*(MAXNAMELENGTH+1));
  strcpy (Name, name);
  sf->Name = Name;
  sf->desc = desc;
  for (i = 0; i < 3; i++)
    size[i] = desc->gncell[i];
  Field = ptr;
  for (i = 0; i < n; i++) {
    sf->Field[i] = Field+i*size[0]*size[1]*size[2]; /* The fields are contiguous in memory */
  }
  return sf;
}

void FreeMultiScalarField (sf)
     MultiScalarField *sf;
{
  free (sf->Name);
  free (sf);
}

VectorField *CreateVectorField (desc, name, ptr)
     tGrid_CPU *desc;
     char *name;
     real *ptr;
{
  char *Name;
  char string[MAXLINELENGTH];
  VectorField *vf;
  long size[3], i;
  real *Field;
  vf = prs_malloc (sizeof(VectorField));
  if (strlen(name) > MAXNAMELENGTH) {
    sprintf (string, "%s is longer than the %ld characters limit.", name, MAXNAMELENGTH);
    prs_error (string);
  }
  Name = prs_malloc(sizeof(char)*(MAXNAMELENGTH+1));
  strcpy (Name, name);
  vf->Name = Name;
  vf->desc = desc;
  for (i = 0; i < 3; i++)
    size[i] = desc->gncell[i];
  Field = ptr;
  for (i = 0; i < NDIM; i++) {
    vf->Field[i] = Field+i*size[0]*size[1]*size[2]; /* The velocity fields are contiguous in memory */
   }
  return vf;
}
  
void FreeVectorField (vf)
     VectorField *vf;
{
  free (vf->Name);
  free (vf);
}

InterfaceFlux *CreateInterfaceFlux (desc)
     tGrid_CPU *desc;
{
  InterfaceFlux *iflux;
  real *field[3][2];
  long dim, side, dim1, dim2, size[3];
  iflux = (InterfaceFlux *)prs_malloc (sizeof(InterfaceFlux));
  iflux->desc = desc;
  for (dim = 0; dim < 3; dim++)	/* Do not change this 3 (value 1 is
				   needed below for extra
				   dimensions) */
    size[dim] = desc->gncell[dim];
  for (dim = 0; dim < NDIM; dim++) {
    dim1 = (dim == 0);
    dim2 = 2 - (dim == 2);
    for (side = INF; side <= SUP; side++) {
      field[dim][side] = prs_malloc(sizeof(real)*size[dim1]*size[dim2]);
      iflux->Flux[dim][side] = field[dim][side];
    }
  }
  return iflux;
}

void FreeInterfaceFlux (iflux)
     InterfaceFlux *iflux;
{
  long dim, side;
  for (dim = 0; dim < NDIM; dim++) {
    for (side = INF; side <= SUP; side++) {
      free (iflux->Flux[dim][side]);
    }
  }
  free (iflux);
}

PressureFaces *CreatePressureFaces (desc)
     tGrid_CPU *desc;
{
  PressureFaces *pres;
  real *field[3][2];
  long dim, side, dim1, dim2, size[3];
  pres = (PressureFaces *)prs_malloc (sizeof(PressureFaces));
  pres->desc = desc;
  for (dim = 0; dim < 3; dim++)	/* Do not change this 3 (value 1 is
				   needed below for extra
				   dimensions) */
    size[dim] = desc->gncell[dim];
  for (dim = 0; dim < NDIM; dim++) {
    dim1 = (dim == 0);
    dim2 = 2 - (dim == 2);
    for (side = INF; side <= SUP; side++) {
      field[dim][side] = prs_malloc(sizeof(real)*size[dim1]*size[dim2]);
      pres->Pressure[dim][side] = field[dim][side];
    }
  }
  return pres;
}

void FreePressureFaces (pres)
     PressureFaces *pres;
{
  long dim, side;
  for (dim = 0; dim < NDIM; dim++) {
    for (side = INF; side <= SUP; side++) {
      free (pres->Pressure[dim][side]);
    }
  }
  free (pres);
}

FluidPatch *CreateFluidPatch (desc, name)
     tGrid_CPU *desc;
     char *name;
{
  char *Name;
  real *StartField;
  long dim, i, size[3], Size;
  ScalarField *Density, *Energy, *Potential;
  MultiScalarField *Delta;
  VectorField *Velocity;
  InterfaceFlux *MassFlux, *MomentumFlux[3], *EnergyFlux;
  PressureFaces *Pressure;
  FluidPatch *patch;
  patch = prs_malloc (sizeof(FluidPatch));
  patch->desc = desc;
  if (strlen(name) > MAXNAMELENGTH) 
    prs_error ("%s is longer than the %ld characters limit.", name, MAXNAMELENGTH);
  Name = prs_malloc(sizeof(char)*(MAXNAMELENGTH+1));
  strcpy (Name, name);
  patch->Name = Name;
  for (i = 0; i < 3; i++)
    size[i] = desc->gncell[i];
  Size = size[0] * size[1] * size[2];
  StartField = prs_malloc (sizeof(real)*Size*(1+1+NDIM+NDimSplit+1));
  /* Global contiguous allocation */
  patch->StartField = StartField;
  Density = CreateScalarField (desc, "density", StartField);
  patch->Ptr[_Density_] = Density->Field;
  Energy = CreateScalarField (desc, "energy", StartField+Size);
  patch->Ptr[_Energy_] = Energy->Field;
  Velocity = CreateVectorField (desc, "velocity", StartField+2*Size);
  Delta = CreateMultiScalarField (desc, "delta", NDimSplit, StartField+(2+NDIM)*Size);
  patch->Delta = Delta;
  Potential = CreateScalarField (desc, "potential", StartField+(2+NDIM+NDimSplit)*Size);
  patch->Ptr[_Potential_] = Potential->Field;
  for (i = 0; i < NDimSplit; i++)
    patch->Ptr[_Delta_+i] = Delta->Field[i];
  patch->Ptr[_Velocity_] = Velocity->Field[0];
  patch->Ptr[_Velocity_+1] = Velocity->Field[1];
  patch->Ptr[_Velocity_+2] = Velocity->Field[2];
  MassFlux = CreateInterfaceFlux (desc);
  EnergyFlux = CreateInterfaceFlux (desc);
  for (dim = 0; dim < NDIM; dim++)
    MomentumFlux[dim] = CreateInterfaceFlux (desc);
  Pressure = CreatePressureFaces (desc);
  patch->Pressure = Pressure;
  patch->Density = Density;
  patch->Energy = Energy;
  patch->Velocity = Velocity;
  patch->MassFlux = MassFlux;
  patch->EnergyFlux = EnergyFlux;
  for (dim = 0; dim < NDIM; dim++)
    patch->MomentumFlux[dim] = MomentumFlux[dim];
  patch->Potential = Potential;
  patch->PotentialSet = FALSE;
  return patch;
}

void FreeFluidPatch (patch)
     FluidPatch *patch;
{
  long dim;
  free (patch->Name);
  FreeScalarField (patch->Density);
  FreeScalarField (patch->Energy);
  FreeMultiScalarField (patch->Delta);
  FreeVectorField (patch->Velocity);
  FreeInterfaceFlux (patch->MassFlux);
  FreeInterfaceFlux (patch->EnergyFlux);
  FreePressureFaces (patch->Pressure);
  for (dim = 0; dim < NDIM; dim++)
    FreeInterfaceFlux (patch->MomentumFlux[dim]);
  FreeScalarField (patch->Potential);
  free (patch->StartField);
  free (patch);
}

  
