#include "mhd.h"

void Godunov_UnsplitUpdate ()
{
  FluidWork *fp;
  long gncell[3], stride[3], i, j, k, idx[3], size[3];
  long m_other_side, m, mp, idm, mdim;
  long ip1, ip2, id;
  real *momentum[3], *Velocity[3], *Density, *InvVolume, *Velf;
  real *energy, *energy_tot;
  real *_radius, *_colatitude;
  real delta_mass, delta_mom;
  //  real delta_energy, kinetic, vlin[3];
  real *inter[3], radius, *ipres[3];
  fp = CurrentFluidPatch;
  InvVolume = fp->desc->InvVolume;
  getgridsize (fp->desc, gncell, stride);
  for (i = 0; i < NDIM; i++) {
    inter[i] = fp->desc->InterSurface[i];
    ipres[i] = fp->GodPressure[i];
    momentum[i] = fp->Momentum[i];
    Velocity[i] = fp->Velocity[i];
    size[i] = gncell[i]-2*Nghost[i];
  }
  _radius = fp->desc->Center[_Radial_];
  _colatitude = fp->desc->Center[_Colatitude_];
  energy = fp->Energy;
  energy_tot = fp->Energy_tot;
  Density = fp->Density;	/* In this loop the momentum[3]  array
				   is   filled; if appropriate one  of
				   these values represents the angular
				   momentum */
  /* In the non isothermal case we also fill the total energy (internal+kinetic) for each zone */
  for (idm = 0; idm < NDIM; idm++) {
    Velf = Velocity[idm];
    for (k = Nghost[2]; k < gncell[2]-Nghost[2]; k++) {
      for (j = Nghost[1]; j < gncell[1]-Nghost[1]; j++) {
	for (i = Nghost[0]; i < gncell[0]-Nghost[0]; i++) {
	  m = i*stride[0]+j*stride[1]+k*stride[2];
	  //vlin[idm] = Velocity[idm][m];
	  //momentum[idm][m] = Density[m]*vlin[idm];
	  momentum[idm][m] = Density[m]*Velf[m];
	  if ((CoordType == CYLINDRICAL) && (idm == _Azimuthal_)) {
	    momentum[idm][m] = Density[m]*(Velf[m]+OMEGAFRAME);
	    radius = _radius[m];
	    momentum[idm][m]*= radius*radius;
	    //vlin[idm] = radius*(vlin[idm]+OMEGAFRAME);
	  }
	  if ((CoordType == SPHERICAL) && (idm == _Azimuthal_)) {
	    momentum[idm][m] = Density[m]*(Velf[m]+OMEGAFRAME);
	    radius = _radius[m];
	    radius *= sin(_colatitude[m]);
	    momentum[idm][m]*= radius*radius;
	    //vlin[idm] = radius*(vlin[idm]+OMEGAFRAME);
	  }
	  if ((CoordType == SPHERICAL) && (idm == _Colatitude_)) { /* The colatitude velocity is angular */
	    radius = _radius[m];
	    momentum[idm][m]*= radius*radius;
	    //vlin[idm] *= radius;
	  }
	}
      }
    }
  }
//	if (!Isothermal) {	/* The kinetic energy below is evaluated in an inertial frame */
//	  kinetic = 0.0;
//	  for (idm = 0; idm < NDIM; idm++)
//	    kinetic += vlin[idm]*vlin[idm];
//	  energy_tot[m] = energy[m]+Density[m]*.5*kinetic;	/* Volumic total energy */
//	}
  for (mdim = 0; mdim < NDIM; mdim++) {
    for (k = Nghost[2]; k < gncell[2]-Nghost[2]; k++) {
      /* In this  loop the  momenta and densities  are updated  from the
	 fluxes given by the Riemann solver, as well as the total energy
	 in the non isothermal version */
      for (j = Nghost[1]; j < gncell[1]-Nghost[1]; j++) {
	for (i = Nghost[0]; i < gncell[0]-Nghost[0]; i++) { 
	  delta_mom = 0.0;
	  m = i*stride[0]+j*stride[1]+k*stride[2];
	  for (idm = 0; idm < NDIM; idm++) {
	    m_other_side = m+stride[idm];
	    delta_mom+= fp->GodF_mom[mdim][idm][m];
	    delta_mom-= fp->GodF_mom[mdim][idm][m_other_side];
	  }
	  momentum[mdim][m] += delta_mom*InvVolume[m];
	}
      }
    }
  }
  /* In this loop the densities are updated from the fluxes given by
     the  Riemann solver, as  well as  the total  energy in  the non
     isothermal version */
  for (k = Nghost[2]; k < gncell[2]-Nghost[2]; k++) {
    for (j = Nghost[1]; j < gncell[1]-Nghost[1]; j++) {
      for (i = Nghost[0]; i < gncell[0]-Nghost[0]; i++) { 
	m = i*stride[0]+j*stride[1]+k*stride[2];
	// delta_energy = 0.0;
	delta_mass=0.0;
	for (idm = 0; idm < NDIM; idm++) {
	  m_other_side = m+stride[idm];
	  delta_mass+= fp->GodF_mass[idm][m];
	  delta_mass-= fp->GodF_mass[idm][m_other_side];
	//	    if (!Isothermal) {
	//	      delta_energy+= fp->GodF_energy[idm][m];
	//	      delta_energy-= fp->GodF_energy[idm][m_other_side];
	//	    }
	}
	Density[m] += delta_mass*InvVolume[m];
	//	  if (!Isothermal)
	//	    energy_tot[m] += delta_energy*InvVolume[m];
      }
    }
  }
  for (idm = 0; idm < NDIM; idm++) {
    ip1 = (idm == 0);	/* If yes then keep track of flux and interface pressure */
    ip2 = 2-(idm == 2);	/* in order to ensure conservation across nested grids */
    for (k = Nghost[2]; k < gncell[2]-Nghost[2]; k++) {
      idx[2] = k;
      for (j = Nghost[1]; j < gncell[1]-Nghost[1]; j++) {
	idx[1] = j;
	for (i = Nghost[0]; i < gncell[0]-Nghost[0]; i++) {
	  idx[0] = i;
	  m = i*stride[0]+j*stride[1]+k*stride[2];
	  //	  vlin[idm] = Velocity[idm][m] = momentum[idm][m]/Density[m];
	  if ((CoordType == CYLINDRICAL) && (idm == _Azimuthal_)) {
	    radius = _radius[m];
	    Velocity[idm][m] = momentum[idm][m]/(radius*radius);
	    Velocity[idm][m] /= Density[m];
	    Velocity[idm][m] -= OMEGAFRAME;
	    //  vlin[idm] = (Velocity[idm][m]+OMEGAFRAME)*radius;
	  } else if ((CoordType == SPHERICAL) && (idm == _Azimuthal_)) {
	    radius = _radius[m];
	    radius *= sin(_colatitude[m]);
	    Velocity[idm][m] = momentum[idm][m]/(radius*radius);
	    Velocity[idm][m] /= Density[m];
	    Velocity[idm][m] -= OMEGAFRAME;
	    //	    vlin[idm] = (Velocity[idm][m]+OMEGAFRAME)*radius;
	  } else if ((CoordType == SPHERICAL) && (idm == _Colatitude_)) { /* Colatitude velocity is angular */
	    Velocity[idm][m] = momentum[idm][m]/Density[m];
	    radius = _radius[m];
	    Velocity[idm][m] /= (radius * radius);
	    //	    vlin[idm] = Velocity[idm][m]*radius;
	  } else
	    Velocity[idm][m] = momentum[idm][m]/Density[m];
//	if (!Isothermal) {	/* The kinetic energy below is evaluated in an inertial frame */
//	  kinetic = 0.0;
//	  for (idm = 0; idm < NDIM; idm++)
//	    kinetic += vlin[idm]*vlin[idm];
//	  energy[m] = energy_tot[m]-.5*kinetic*Density[m];
//	}
	  if (idx[idm] == Nghost[idm]) {	/* Are we on the lower face ? */
	    id  = (idx[ip1]-Nghost[ip1])+(idx[ip2]-Nghost[ip2])*size[ip1]; 
	    fp->Fluid->MassFlux->Flux[idm][INF][id] += fp->GodF_mass[idm][m];
	    //	    if (!Isothermal)
	    //	      fp->Fluid->EnergyFlux->Flux[idm][INF][id] += fp->GodF_energy[idm][m];
	    fp->Fluid->Pressure->Pressure[idm][INF][id] += ipres[idm][m]*inter[idm][m];
	    for (mdim = 0; mdim < NDIM; mdim++)
	      fp->Fluid->MomentumFlux[mdim]->Flux[idm][INF][id] += fp->GodF_mom[mdim][idm][m];
	  }
	  if (idx[idm] == size[idm]+Nghost[idm]-1) { /* Are we on the upper face ? */
	    ip1 = (idm == 0);
	    ip2 = 2-(idm == 2);
	    mp = m+stride[idm];
	    id  = (idx[ip1]-Nghost[ip1])+(idx[ip2]-Nghost[ip2])*size[ip1];
	    fp->Fluid->MassFlux->Flux[idm][SUP][id] += fp->GodF_mass[idm][mp];
	    if (!Isothermal)
	      fp->Fluid->EnergyFlux->Flux[idm][SUP][id] += fp->GodF_energy[idm][mp];
	    fp->Fluid->Pressure->Pressure[idm][SUP][id] += ipres[idm][mp]*inter[idm][mp];
	    for (mdim = 0; mdim < NDIM; mdim++)
	      fp->Fluid->MomentumFlux[mdim]->Flux[idm][SUP][id] += fp->GodF_mom[mdim][idm][mp];
	  }
	}
      }
    }
  }
}
