struct tgrid {
  long level;			/* Refinement level */
  long ncell[3];		/* Number of zones in each dimension (excl. ghosts) */
  long gncell[3];		/* Number of zones in each dimension (incl. ghosts) */
  long nsize[3];		/* Absolute size (excl. ghosts) */
  long ncorner_min[3];		/* Absolute position (min corner, excl. ghosts) */
  long ncorner_max[3];		/* Absolute position (max corner, excl. ghosts) */
  long gnsize[3];		/* Absolute size (incl. ghosts) */
  long gncorner_min[3];		/* Absolute position (min corner, incl. ghosts) */
  long gncorner_max[3];		/* Absolute position (max corner, incl. ghosts) */
  real size[3];			/* Size in physical units */
  real corner_min[3];		/* Position of min corner in physical units */
  real corner_max[3];		/* Position of max corner in physical units */
  real *Edges[3];		/* Interface position in each dimension */
  boolean present;		/* True if and only if that grid is mapped */
				/* by local CPU */
  long number;			/* Logical number of grid */
  struct tgrid *next;		/* Pointer to next grid */
  struct tgrid *prev;		/* Pointer to previous grid */
  long BoundaryConditions[3][2];/* List of BC's on faces */
  long minCPU;			/* range of processes over which */
  long maxCPU;			/* the grid is distributed */
  long dn[3];			/* Cell size in absolute units */
  long linenumber;		/* Line number in grid file */
  long monoCPU;			/* Information used for a restart only */
};

typedef struct tgrid tGrid;
typedef struct fluidpatch FluidPatch;

struct tgrid_cpu {
  long   parent;       		/* Logical number of "parent" grid */
  long   number;		/* Internal logical number */
  tGrid *Parent;		/* Pointer to "parent */
  long   level;			/* Refinement level */
  int    cpu;			/* process number of corresponding CPU grid */
  long   ncell[3];		/* Number of zones in each dimension (excl. ghosts)*/
  long   gncell[3];		/* Number of zones in each dimension (incl. ghosts)*/
  long   stride[3];		/* Data storage stride in each dimension */
  long   nsize[3];		/* Absolute size */
  long iface[3][2];		/* Interface type (CPU->brother, True BC, Mesh->other mesh) */
  long ncorner_min[3];		/* Absolute position (min corner) */
  long ncorner_max[3];		/* Absolute position (max corner) */
  long pcorner_min[3];		/* Position in parent (in cells) - min */
  long pcorner_max[3];		/* Position in parent (in cells) - max */
  long gnsize[3];		/* Absolute size (incl. ghosts) */
  long gncorner_min[3];		/* Absolute position (min corner, incl. ghosts) */
  long gncorner_max[3];		/* Absolute position (max corner, incl. ghosts) */
  real size[3];			/* Size in physical units */
  real corner_min[3];		/* Position of min corner in physical units */
  real corner_max[3];		/* Position of max corner in physical units */
  real *InvVolume;		/* Inverse of volume of each cell */
  real *Center[3];		/* Center of zones in each dimension */
  real *Metric[3][2];		/* 1D Arrays for metric coefficients */
  real *InvMetric[3][2];	/* 1D Arrays for metric coefficients */
  real *MomCorrRatioL[3][3];
  real *MomCorrRatioR[3][3];
  real *InterSurface[3];	/* Surface of interfaces in each dimension */
  real *Edges[3];		/* Interface position in each dimension */
  char *Hidden;			/* A set of flags which says if the zone lies behind another */
  char *CommSource;		/* A set of bit flags used by LeVeque method */
  char *CommDest;		/* A set of bit flags used by LeVeque method */
  char *SrcCommPerp;		/* A set of bit flags to switch between LeVeque/Keplerian */
  char *DestCommPerp;		/* A set of bit flags to switch between LeVeque/Keplerian */
  FluidPatch *Fluid;		/* Pointer to fluid patch */
  struct tgrid_cpu *next;	/* Pointer to next CPU Grid */
  struct tgrid_cpu *prev;	/* Pointer to previous CPU Grid */
};

typedef struct tgrid_cpu tGrid_CPU;

struct communicator {
  int tag;			/* Necessarily 'int' and not 'long' as
				   this has to correspond to MPI
				   implementation */
  long imin_dest[3],imax_dest[3];
  long imin_src[3],imax_src[3];
  long dest_level;
  long src_level;
  long grid_src, grid_dest;
  long facedim;
  long faceside;
  long CPU_src, CPU_dest;
  long nb_src, nb_dest;
  long Imin[3], Imax[3];
  long type;			/* Should be one of those: GHOST, MEAN, FLUX */
  real *buffer;
  tGrid_CPU *srcg, *destg;
  long size;
  struct communicator *next, *prev;
};

struct commhash {
  struct communicator *com;
  struct commhash *next;
  struct commhash *prev;
};

typedef struct communicator Communicator;

typedef struct commhash CommHash;

struct scalarfield {
  char *Name;
  real *Field;
  tGrid_CPU *desc;
};

typedef struct scalarfield ScalarField;

struct multiscalarfield {
  char *Name;
  real *Field[20];
  tGrid_CPU *desc;
};

typedef struct multiscalarfield MultiScalarField;

struct vectorfield {
  char *Name;
  real *Field[3];
  tGrid_CPU *desc;
};

typedef struct vectorfield VectorField;

struct interfaceflux {
  real *Flux[3][2];
  tGrid_CPU *desc;
};

typedef struct interfaceflux InterfaceFlux;

struct pressurefaces {
  real *Pressure[3][2];
  tGrid_CPU *desc;
};

typedef struct pressurefaces PressureFaces;

struct fluidpatch {
  char *Name;
  ScalarField *Density;
  MultiScalarField *Delta;
  real *StartField;
  VectorField *Velocity;
  ScalarField *Energy;
  real *Ptr[20];		/* Short cut to fields */
  InterfaceFlux  *MassFlux;
  InterfaceFlux  *MomentumFlux[3];
  InterfaceFlux  *EnergyFlux;
  PressureFaces  *Pressure;
  ScalarField *Potential;
  tGrid_CPU      *desc;
  boolean PotentialSet;
};

struct fluidwork {
  real *Density;
  real *Density_Pred;
  real *Potential;
  real *Energy;
  real *Energy_tot;
  real *Divergence;
  real *Momentum[3];
  real *Velocity[3];
  real *Velocity_Pred[3];
  real *SourceRhoPred;		/* to store geometric term */
  real *SourceVelocity[3];
  real *Accel[3];
  real *delta[3];
  real *SlopeDensity[3];
  real *SlopeEnergy[3];
  real *SlopeVelocity[3][3];
  real *StressTensor[3];
  real *GodF_mass[3];
  real *GodF_energy[3];
  real *GodF_mom[3][3];		/* right index: dim for face, left index: comp of mom */
  real *GodPressure[3];		/* Godunov Pressure on faces */
  tGrid_CPU *desc;
  FluidPatch *Fluid;
  long Size[3];
  long GroupSize[3];
};

typedef struct fluidwork FluidWork;

struct beam {
  real *rho;
  real *delta;
  real *u;
  real *v_perp[2];
  real *cs;
  real *slope_rho[3];
  real *slope_energy[3];
  real *slope_u[3];
  real *slope_v_perp[2][3];
  real *center;
  real *rawcoord;
  real *edge;
  real *srcrho;
  real *intersurface;
  real *mass_flux;
  real *energy_flux;
  real *momentum_flux[3];
  real *rhoL;
  real *rhoR;
  real *uL;
  real *uR;
  real *eL;
  real *eR;
  real *rho_pred;
  real *u_pred;
  real *v_perp_pred[2];
  real *v_perp_L[2];
  real *v_perp_R[2];
  real *pressure_godunov;
  real *metperp[2];
  real *source;
  real *source_perp[2];
  real *MomCorrRatioL[2];
  real *MomCorrRatioR[2];
  long dim[3];
  long length;
  tGrid_CPU *desc;
  boolean true_bc[2];
  boolean ZoneSplit;
  boolean MomCorr[2];
  real rawcoord1;
  real rawcoord2;
};

typedef struct beam Beam;

struct gridfileinfo {
  long number;
  long linenumber;
  long nc_min[3];
  long nc_max[3];
  long size[3];
  real xmin[3], xmax[3];
  long level;
  long bc[6];
  long monoCPU;			/* Restart information */
  boolean last;
};

typedef struct gridfileinfo GridFileInfo;
