#include "mhd.h"

inline real initialguess (fs, m2, curv)
     real fs, m2, curv;
/* fs corresponds to {\cal F} (fullsource)*/
/* m2 corresponds to {\cal M}^2 (zone Mach number to the square) */
/* curv corresponds to \mu */
{
  return (fs-curv*m2)/(1.0-m2);
}

inline real newtoniterate (delta, fs, m2, curv)
     real delta, fs, m2, curv;
{
  real newdelta, d2, iomd2;
  d2 = delta*delta;
  iomd2 = 1./(1.-d2);
  newdelta = delta-(fs-delta+m2*(delta-curv)*iomd2)/\
    (m2*(1.+d2-2.0*delta*curv)*iomd2*iomd2-1.0);
  return newdelta;
}

inline real findrootexact (fs, m2, curv)
     real fs, m2, curv;
{
  real p, q, discr, phi, w, x, delta;
  p = m2-1.0-fs*fs/3.0;
  q = 2.0*fs*fs*fs/27.0-(m2-1.0)*fs/3.0-fs+curv*m2;
  discr = q*q+4.0*p*p*p/27.0;
  if (discr < 0.0) {
    phi = (2.*M_PI+atan2(-sqrt(-discr),q))/3.0;
    delta=sqrt(-p/3.0)*2.0*cos(phi)+fs/3.0;
  } else {
    if (q < 0.0) 		/* This test is meant */
				/* to avoid the w=0 singularity */
      w = .5*(q-sqrt(discr));
    else 
      w = .5*(q+sqrt(discr));
    w = cbrt(w);
    if (w != 0.0)
      x = (w-p/3./w);
    else
      x = 0.0;			/* This should occur iff q=0 AND p=0 */
				/* This last solution has been obtained */
				/* by continuity: p-->0 */
    delta = x+fs/3.0;
  }
  return delta;
}

inline real findroot (fs, m2, curv)
     real fs, m2, curv;
{
  real d1, d2;
  if (fs*fs+m2 < .2) {
    d1 = initialguess (fs, m2, curv);
    do {
      d2 = d1;
      d1 = newtoniterate (d2, fs, m2, curv);
    } while (fabs(d2-d1) > 1e-8);
  } else {
    d1 = findrootexact (fs, m2, curv);
  }
  return d1;
}

