#include "mhd.h"

void plm (beam, dt)
     Beam *beam;
     real dt;
{
  long i, j, k, n;
  real rhoL, rhoR, a, uL, uR, vperpL[2], vperpR[2], mu;
  real dxp, dxm, sa1, sa2, roa, aor, source_vperp[2], K1=0.0, K2;
/* In the expression below, the first index in the dxrho and dxv
   arrays is for the dimension along which the derivative is to be
   taken, while the second in dxv is for which component of the
   velocity one considers (dxv[i][j] = "dv^j/dx^i") */
  real *vperp[2], *dxrho[3], *dxv[3][3], *rho, *u, *cs, *inter, *delta;
  real rho0, u0, da1p, da2p, da1m, da2m, cont_transv, euler_transv;
  boolean zs;

  n   = beam->length;
  u   = beam->u;
  rho = beam->rho;
  cs   = beam->cs;
  inter= beam->intersurface;
  delta= beam->delta;
  for (k = 0; k < NDIM-1; k++)
    vperp[k] = beam->v_perp[k];
  for (k = 0; k < NDIM; k++) {
    dxrho[k] = beam->slope_rho[k];
    dxv[k][0] = beam->slope_u[k];
    dxv[k][1] = beam->slope_v_perp[0][k];
    dxv[k][2] = beam->slope_v_perp[1][k];
  }
  zs = (ZoneSplitting[beam->dim[0]] == TRUE);
  for (i = 1; i < n-1; i++) {	/* Do not change these limits. Note
				   that in case NGHOST>2, they can
				   probably be changed. However this
				   choice is safe whatever the value
				   of NGHOST */
    u0  = u[i];
    a    =  cs[i];
    rho0 = rho[i];
    aor  = a/rho0;
    roa  = rho0/a;
    if (NDIM>1) vperpR[0] = vperpL[0] = vperp[0][i];
    if (NDIM>2) vperpR[1] = vperpL[1] = vperp[1][i];
    cont_transv = 0.0;		/* Perpendicular terms for continuity eq. */
    for (k = 0; k < NDIM-1; k++)
      cont_transv += vperp[k][i]*dxrho[k+1][i]+rho0*dxv[k+1][k+1][i];
    euler_transv = 0.0;		/* Perpendicular terms for Euler eq. */
    for (k = 0; k < NDIM-1; k++)
      euler_transv += vperp[k][i]*dxv[k+1][0][i];
    
    rhoL = rhoR = rho0;
    uL   = uR   = u0;

    dxp  = 2.*(beam->edge[i+1]-beam->center[i]);
    dxm  = 2.*(beam->center[i]-beam->edge[i]);

    rhoL += -.5*dt*cont_transv;	/* One must do this *before* the 'zs' test below */
    rhoR += -.5*dt*cont_transv;

    uL += -.5*dt*euler_transv;
    uR += -.5*dt*euler_transv;
    
    if (zs) {
      mu = (inter[i]-inter[i+1])/(inter[i]+inter[i+1]);
      rhoR *= (1.0-delta[i]);
      uR *=  (1.0-mu)/(1.0-delta[i]);
      rhoL *= (1.0+delta[i]);
      uL *=  (1.0+mu)/(1.0+delta[i]);
      K1 = 4.0*rho0*delta[i]/(dxp+dxm);
      K2 = 4.0*u0/(dxp+dxm)*(mu-delta[i])/(1.0-delta[i]*delta[i]);
      K2 = exp(-K2*dt/2.);
    } else {
      rhoL += -.5*dt*(beam->srcrho[i]);
      rhoR += -.5*dt*(beam->srcrho[i]);
      uL   += +.5*dt*(beam->source[i]);
      uR   += +.5*dt*(beam->source[i]);
      K2 = 1.0;
    }

    sa1  = (dxrho[0][i]+roa*dxv[0][0][i]);
    sa2  = (dxrho[0][i]-roa*dxv[0][0][i]);
    da1p = .25*(dxp-(u0+a)*dt)*sa1;
    da2p = .25*(dxp-(u0-a)*dt)*sa2;
    da1m = .25*(-dxm-(u0+a)*dt)*sa1;
    da2m = .25*(-dxm-(u0-a)*dt)*sa2;
    
    if ((u0+a) >= 0.0) {	/* These tests have to be written like that */
      rhoL += da1p*K2;		/* They must not use e.g. if (u < 0.0) ... else ... */
      uL += da1p*aor*K2;	/* as the result violates right/left symmetry */
    }
    if ((u0+a) <= 0.0) {	/* As a consequence a Keplerian disk with equatorial */
      rhoR += da1m*K2;		/* symmetry would violate this symmetry */
      uR += da1m*aor*K2;
    }
    if ((u0-a) >= 0.0) {
      rhoL += da2p*K2;
      uL += -da2p*aor*K2;
    }
    if ((u0-a) <= 0.0) {
      rhoR += da2m*K2;
      uR += -da2m*aor*K2;
    }
    if (zs) {
      if (u0 >= 0.0) {
	uL +=  .25*aor*aor*K1*dxrho[0][i]*dt*(dxp-u0*dt)*K2;
	rhoL+=-.25*K1*dxv[0][0][i]*dt*(dxp-u0*dt)*K2;
      }
      if (u0 <= 0.0) {
	uR +=  .25*aor*aor*K1*dxrho[0][i]*dt*(-dxm-u0*dt)*K2;
	rhoR+=-.25*K1*dxv[0][0][i]*dt*(-dxm-u0*dt)*K2;
      }
    }
    for (k = 0; k < NDIM-1; k++) {
      source_vperp[k] = -a*aor*dxrho[k+1][i]+beam->source_perp[k][i];
      for (j = 0; j < NDIM-1; j++)
	source_vperp[k] += -vperp[j][i]*dxv[j+1][k+1][i];
      /* Les 6 lignes ci-dessous sont a priori les fautives... Sont-elles les seules ? Non, ce ne sont pas les seules... Faire des tests supplmentaires... */
      if (0.0*u0 >= 0.0) {	/* cuidadin !! */
        	vperpL[k] += .5*dxv[0][k+1][i]*(dxp-u0*dt);
      }
      if (0.0*u0 <= 0.0) {	/* cuidadin !! */
 	vperpR[k] += .5*dxv[0][k+1][i]*(-dxm-u0*dt);
      }
      if (beam->MomCorr[k]) {
        vperpL[k] *= beam->MomCorrRatioL[k][i];
        vperpR[k] *= beam->MomCorrRatioR[k][i];
      }
      vperpL[k] += source_vperp[k]*.5*dt;
      vperpR[k] += source_vperp[k]*.5*dt;
      beam->v_perp_L[k][i+1] = vperpL[k];
      beam->v_perp_R[k][i]   = vperpR[k];
    }
    
    if (rhoL <= _Small_Rho) rhoL = _Small_Rho;
    if (rhoR <= _Small_Rho) rhoR = _Small_Rho;
    
    /* The correction below takes into account the gradient of the
       sound speed along the beam, as a source term on the velocity */

    if (!zs) {
      uL += -.5*dt*(cs[i+1]*cs[i+1]-cs[i]*cs[i])/dxp;
      uR += .5*dt*(cs[i-1]*cs[i-1]-cs[i]*cs[i])/dxm;
    }

    beam->rhoL[i+1] = rhoL;
    beam->rhoR[i] = rhoR;
    beam->uL[i+1] = uL;
    beam->uR[i] = uR;
  }
  AdjustBeamBoundaries (beam);
}
