#include "mhd.h"

void Compute_Fluxes_Iso (beam, dt)
     Beam *beam;
     real dt;
{
  long i,dim,k;
  real rhoL, rhoR, aL, aR, uL, uR, vpL[2], vpR[2];
  real us, ps, S_shock, a_godunov;
  real rho_godunov, u_godunov, vp_godunov[2];
  real mass_flux, pu_flux;
  real surfdt;
  real a2;
  dim = beam->dim[0];
  for (i = Nghost[dim]; i <= beam->length-Nghost[dim]; i++) {
    uL   = beam->uL[i];
    uR   = beam->uR[i];
    for (k = 0; k < NDIM-1; k++) {
      vpL[k] = beam->v_perp_L[k][i];
      vpR[k] = beam->v_perp_R[k][i];
    }
    rhoL = beam->rhoL[i];
    rhoR = beam->rhoR[i];
    //aL   = beam->cs[i-1];
    //  aR   = beam->cs[i];
    aR = aL   = .5*(beam->cs[i-1]+beam->cs[i]);
    a2 = aL * aL;
    __Riemann_Solver (rhoL, rhoR, uL, uR, aL, aR, &us, &ps); /* Call the Riemann solver */
    if (0.0 < us) {
      /* we are in the star or left region or fan */
      for (k = 0; k < NDIM-1; k++)
	vp_godunov[k] = vpL[k];
      a_godunov = aL;
      if (us < uL) {		/* There is a left shock */
	S_shock = uL-sqrt(ps/rhoL);
	if (0.0 < S_shock) {	/* We are at the left of the left shock */
	  u_godunov = uL;
	  rho_godunov = rhoL;
	} else {		/* we are in the star region */
	  u_godunov = us;
	  rho_godunov = ps/a2;
	}
      } else {			/* There is a left rarefaction */
	if (0.0 < uL-aL) {	/* We are in the left region */
	  u_godunov = uL;
	  rho_godunov = rhoL;
	} else {
	  if (0.0 > us-aL) {	/* We are in the star region */
	    u_godunov = us;
	    rho_godunov = ps/a2;
	  } else {		/* We are in the fan */
	    u_godunov = aL;
	    rho_godunov = rhoL*exp((uL-u_godunov)/aL);
	  }
	}
      }
    } else {			/* We are in the star or right region or fan */
      for (k = 0; k < NDIM-1; k++)
	vp_godunov[k] = vpR[k];
      a_godunov = aR;
      if (us > uR) {		/* There is a right shock */
	S_shock = uR+sqrt(ps/rhoR); 
	if (0.0 > S_shock) {	/* we are at the right of the right shock */
	  u_godunov = uR;
	  rho_godunov = rhoR;
	} else {		/* we are in the star region */
	  u_godunov = us;
	  rho_godunov = ps/a2;
	}
      } else {			/* There is a right rarefaction */
	if (0.0 > uR+aR) {	/* we are in the right part */
	  u_godunov = uR;
	  rho_godunov = rhoR;
	} else {
	  if (0.0 < us+aR) {	/* we are in the star region */
	    u_godunov = us;
	    rho_godunov = ps/a2;
	  } else {		/* we are in the fan */
	    u_godunov = -aR;
	    rho_godunov = rhoR*exp(-(uR-u_godunov)/aR);
	  }
	}
      }
    }
    mass_flux = rho_godunov*u_godunov;
    pu_flux   = rho_godunov*(a2+u_godunov*u_godunov);
    surfdt = beam->intersurface[i] * dt;
    beam->mass_flux[i] = mass_flux * surfdt;
    beam->momentum_flux[0][i] = pu_flux * surfdt;
    for (k = 0; k < NDIM-1; k++)
      beam->momentum_flux[k+1][i] = rho_godunov*vp_godunov[k]*u_godunov * surfdt;
    beam->pressure_godunov[i] = rho_godunov*a2;
  }
}
